import numpy as np
import matplotlib.pyplot as plt
from scipy import interpolate
from scipy.optimize import fsolve
from astropy.time import Time
from astropy.coordinates import get_sun
from astropy.coordinates import SkyCoord
import sys, re
import polarlight_func as pl
import struct, binascii
import sympy as sy
reload(pl)

def ang_solve(ra1,dec1,ra2):
    dec2 = np.deg2rad(np.arange(-90.0,90.0,0.1))
    sep = pl.angular_distance(ra1,dec1,ra2,dec2)
    tt = np.abs(sep-np.deg2rad(18))
    return dec2[np.argmin(tt)]

# given the Y+ RA DEC, calculate the possible Z+ RA DEC
def possible_star_tracker_pointings(target_ra, target_dec, c_sun, earth_ra, earth_dec):
    
    # RA can be any value from 0 - 360 deg
    possible_st_ra = np.deg2rad(np.arange(0.0, 360.0, 1.0))
    # star_tracker angular_distance with target is 18 degree
    # cos(18degree) = sin(dec1)sin(dec2)+cos(dec1)cos(dec2)cos(ra1-ra2)
    possible_st_dec = np.zeros(360)
    for i in range(360):
        possible_st_dec[i] = ang_solve(target_ra,target_dec,possible_st_ra[i])
    
    # number of earth/sun positions, which vary with time
    n_st = len(possible_st_ra)
    n = np.size(earth_ra)
    st_sun_angle = np.zeros([n_st, n])
    st_earth_angle = np.zeros([n_st, n])
    print("Calculating the angles between the star tracker and the Sun/Earth.")
    for i in range(n_st):
        if (i % 60 == 0) or (i == n_st - 1):
            print("   processing ... %d/%d" % (i + 1, n_st))
        st_sun_angle[i, :] = pl.angular_distance(possible_st_ra[i], possible_st_dec[i], c_sun.ra.radian, c_sun.dec.radian)
        st_earth_angle[i, :] = pl.angular_distance(possible_st_ra[i], possible_st_dec[i], earth_ra, earth_dec)

    return possible_st_ra, possible_st_dec, st_sun_angle, st_earth_angle

def possible_star_tracker_pointings_for_grid_new(target_ra, target_dec, c_sun, earth_ra, earth_dec):
    
    target_pos_xyz = np.array([pl.radec_to_xyz(target_ra, target_dec)])
    target_pos_in_sat_xyz = np.array([0., 0., -1.])
    #rol_mat = np.dot(target_pos_in_sat_xyz.T, target_pos_xyz)
    target_in_sat_ra,target_in_sat_dec = pl.xyz_to_radec(0., 0., -1.)
    
    mid = np.cross(target_pos_xyz,target_pos_in_sat_xyz)
    mid = mid / np.linalg.norm(mid)
    w = pl.angular_distance(target_ra,target_dec,target_in_sat_ra,target_in_sat_dec)
    quat0 = pl.rotation_to_quaternion(mid,w)
    
    rot_angle = np.deg2rad(np.arange(0,360,1))
    
    possible_st_ra = np.zeros(360)
    possible_st_dec = np.zeros(360)
    #possible_my_ra = np.zeros(360)
    #possible_my_dec = np.zeros(360)
    
    #mz_xyz = np.array([0., 0., -1.])
    st_xyz = np.array([0., np.sin(np.deg2rad(18)), -np.cos(np.deg2rad(18))])
    
    for i in range(360):
        quat = pl.rotation_to_quaternion(target_pos_xyz,rot_angle[i])
        quat_r = pl.quaternion_product(quat0,quat)
        '''
        rot_matrix = pl.quaterion_to_rotation_matrix(quat[0],quat[1],quat[2],quat[3])
        st_t_xyz = np.matmul(np.matmul(st_xyz, rol_mat), rot_matrix)
        y_t_xyz = np.matmul(np.matmul(y_xyz, rol_mat), rot_matrix)
        '''
        rot_matrix = pl.quaterion_to_rotation_matrix(quat_r[0],quat_r[1],quat_r[2],quat_r[3])
        st_t_xyz = np.matmul(st_xyz, rot_matrix)
        #mz_t_xyz = np.matmul(mz_xyz, rot_matrix)
        possible_st_ra[i], possible_st_dec[i] = pl.xyz_to_radec(st_t_xyz[0], st_t_xyz[1], st_t_xyz[2])
        #possible_mz_ra[i], possible_mz_dec[i] = pl.xyz_to_radec(mz_t_xyz[0], mz_t_xyz[1], mz_t_xyz[2])

    
    # number of earth/sun positions, which vary with time
    n_st = len(possible_st_ra)
    n = np.size(earth_ra)
    st_sun_angle = np.zeros([n_st, n])
    st_earth_angle = np.zeros([n_st, n])
    print("Calculating the angles between the star tracker and the Sun/Earth.")
    for i in range(n_st):
        if (i % 60 == 0) or (i == n_st - 1):
            print("   processing ... %d/%d" % (i + 1, n_st))
        st_sun_angle[i, :] = pl.angular_distance(possible_st_ra[i], possible_st_dec[i], c_sun.ra.radian, c_sun.dec.radian)
        st_earth_angle[i, :] = pl.angular_distance(possible_st_ra[i], possible_st_dec[i], earth_ra, earth_dec)

    return possible_st_ra, possible_st_dec, st_sun_angle, st_earth_angle


def select_star_tracker_radec(possible_st_ra, possible_st_dec, st_sun_angle, st_earth_angle, i_target_on, i_target_off):
    SUN_AVOID_ANGLE = np.deg2rad(60.0)
    EARTH_AVOID_ANGLE = np.deg2rad(100.0)

    q_allowed = (st_sun_angle > SUN_AVOID_ANGLE) & (st_earth_angle > EARTH_AVOID_ANGLE)

    # number of directions(RA/Dec) of the star tracker
    n_st = len(possible_st_ra)
    # number of intervals to observe the target
    n_obs = len(i_target_on)


    st_ra = []
    st_dec = []
    st_time_index = []
    for i in range(n_obs):

        # find common RA/Dec for the star tracker
        q_obs = range(i_target_on[i], i_target_off[i] + 1)
        n_t = i_target_off[i] - i_target_on[i] + 1
        qq = q_allowed[:, q_obs]
        #plt.imshow(qq)
        t = np.sum(qq, 1) == n_t
        if t.sum() > 0:
            d = np.where(t)[0][0]
            # ea = pl.angular_distance(st_ra[d], st_dec[d], earth_ra, earth_dec)
            ea = st_earth_angle[d, q_obs]
            print("      There is a single, good star tracker pointing in obs %d" % i)
            print("      Earth aspect angle, max = %0.2f, min = %0.2f, median = %0.2f" %
                  (np.rad2deg(ea.max()), np.rad2deg(ea.min()), np.rad2deg(np.median(ea))))
            t_i0, t_i1 = pl.find_continous_intervals(t)
            tn = np.where(t)[0]
            if len(t_i0)>1:
                
                mid = (t_i1[0]+360+t_i0[1])/2
                if mid > 359:
                    mid -= 360
                '''
                if i%2==0:
                    mid = t_i1[0]
                else:
                    mid = t_i0[1]
                '''
            else:
                
                mid = (tn[0]+tn[-1])/2
                '''
                if i%2==0:
                    mid = tn[0]
                else:
                    mid = tn[-1]
                '''
            if not t[mid]:
                mid = tn[0]
            i_left = mid
            i_right = mid
        else:
            print("      There is no single, good star tracker pointing in obs %d" % i)
            t0 = q_allowed[:, i_target_on[i]]
            t1 = q_allowed[:, i_target_off[i]]
            t0_i0, t0_i1 = pl.find_continous_intervals(t0)
            t1_i0, t1_i1 = pl.find_continous_intervals(t1)
            
            q_len = np.zeros(360)
            for j in range(360):
                q_len[j] = np.sum(qq[j])
            
            '''
            # find the mid-point
            if (len(t0_i0) > 1) or (len(t1_i0) > 1):
                if len(t0_i0) > 1:
                    ttt = np.array(t0_i1) - np.array(t0_i0)
                    i0 = t0_i0[ttt.argmax()]
                    i1 = t0_i1[ttt.argmax()]
                    t0_i0 = []
                    t0_i0.append(i0)
                    t0_i1 = []
                    t0_i1.append(i1)
                elif len(t1_i0) > 1:
                    ttt = np.array(t1_i1) - np.array(t1_i0)
                    i0 = t1_i0[ttt.argmax()]
                    i1 = t1_i1[ttt.argmax()]
                    t1_i0 = []
                    t1_i0.append(i0)
                    t1_i1 = []
                    t1_i1.append(i1)
               ''' 
                # print('There are multiple allowed segments. please modify the codes to consider that.')    
                # sys.exit('There are multiple allowed segments. please modify the codes to consider that.')
    
            # the larger the delta_i is, the greater the angle is different from 92 degrees
            delta_i = 30
            '''
            # if the left allowed region is on top (small)
            if (abs(t0_i0[0] - t1_i1[0]) > abs(t0_i1[0] - t1_i0[0])):
                mid = int((t0_i1[1] + t1_i0[1]) / 2.)
                i_left = mid - delta_i
                i_right = mid + delta_i
            else:
                mid = int((t0_i0[0] + t1_i1[0]) / 2.)
                i_left = mid + delta_i
                i_right = mid - delta_i
            
            '''
            # if the left allowed region is on top (small)
            mid = int((t0_i1[0] + t1_i0[0]) / 2.)
            i_left = mid - delta_i
            i_right = mid + delta_i
            if i_right >= 360:
                i_right = i_right -360
            '''
            # if the right allowed region is on top , start with lower one
            mid = int((t0_i1[0] + t1_i0[1]) / 2.)
            i_left = mid - delta_i
            i_right = mid + delta_i
            if i_right >= 360:
                i_right = i_right -360
            
            #i_longest = 
            # Crab August
            mid = int((t0_i0[1] + t1_i1[0]) / 2.)
            i_left = mid + delta_i
            i_right = mid - delta_i
            if i_left >= 360:
                i_left = i_left -360
               
            mid = int((t0_i0[0] + t1_i1[0]) / 2.)
            i_left = mid + delta_i
            i_right = mid - delta_i
            if i_left >= 360:
                i_left = i_left -360
            '''
        # start index of the left segment
        i_left_start_time = i_target_on[i]

        

        st_ra.append(possible_st_ra[i_left])
        st_dec.append(possible_st_dec[i_left])
        st_time_index.append(i_left_start_time)
        if (i_right!=i_left):
            # start index of the right segment
            #i_right_start_time = i_target_on[i] + np.where(qq[i_right, :])[0][0]
            i_right_start_time = i_target_on[i] + np.where(qq[i_left, :])[0][-1]
            st_ra.append(possible_st_ra[i_right])
            st_dec.append(possible_st_dec[i_right])
            st_time_index.append(i_right_start_time)
        '''
        plt.imshow(qq)
        x = np.where(qq[i_left, :])[0]
        plt.plot(x, x * 0. + i_left)
        x = np.where(qq[i_right, :])[0]
        plt.plot(x, x * 0. + i_right)
        plt.show()
        '''
    return st_ra, st_dec, st_time_index


def calcualte_qparm(target_ra, target_dec, st_ra, st_dec):
    qparms = []
    for i in range(len(st_ra)):
        qparms.append(pl.construct_quaternion(target_ra, target_dec, st_ra[i], st_dec[i]))

    return qparms

def attitude_assembly(qparms, st_time_index, sun_tracking_time_index):
    # combine inertial points and sun-tracking, sorted by time
    att_time_index = sorted(st_time_index + sun_tracking_time_index)

    att_qparms = []
    k = 0
    for i in att_time_index:
        if i in st_time_index:
            att_qparms.append(qparms[k])
            k += 1
        else:
            att_qparms.append(0) # 0 means sun-tracking

    return att_time_index, att_qparms

def star_tracker_earth_sun_angle(qparms, q_time_index, earth_ra, earth_dec, sun_ra, sun_dec):
    n = len(earth_ra)
    n_qparm = len(qparms)
    st_o_car = np.array([0., np.sin(np.deg2rad(18)), -np.cos(np.deg2rad(18))])

    # output time series
    st_earth_angle = np.zeros(n)
    st_sun_angle = np.zeros(n)


    for i in range(n_qparm):

        q = qparms[i]

        if np.size(q) == 4: # for inertial pointing
            R = pl.quaterion_to_rotation_matrix(q[0], q[1], q[2], q[3])

            st_car = np.matmul(R, st_o_car)
            st_ra, st_dec = pl.xyz_to_radec(st_car[0], st_car[1], st_car[2])

            j0 = q_time_index[i]
            if i < n_qparm - 1:
                j1 = q_time_index[i+1]
            else:
                j1 = n
            st_earth_angle[j0:j1] = pl.angular_distance(st_ra, st_dec, earth_ra[j0:j1], earth_dec[j0:j1])
            st_sun_angle[j0:j1] = pl.angular_distance(st_ra, st_dec, sun_ra[j0:j1], sun_dec[j0:j1])

    return st_earth_angle, st_sun_angle

# given the quaternion and time, generate the commands txt file
def generate_att_cmd(orb_time_bj, att_time_index, att_qparms):
    
    ####test orbit(to be removed)####
    test_t = 80000  #####(to be removed)
    ######(to be removed)
    
    # time delay from the Sun to inertial pointing
    #sec_delay_from_sun_to_iner = -360.
    sec_delay_from_sun_to_iner_new = -480.

    # get time in units of seconds
    t = orb_time_bj.unix
    t0 = t[0] + 30.0 # t0 is the time to execute the command list, by default, it is 00:00:10

    cmd_sec = []  # run time in unit of seconds, starting from zero
    cmd = [] # command
    par = [] # parameter

    # The following commands run once at the beginning of every day
    # failed upgrade in 2019 Dec
    '''
    cmd_sec.append(t0)
    cmd.append('disable_auto_tg_read')
    par.append('')
    
    cmd_sec.append(t0)
    cmd.append('load_bin_file')
    par.append('ty_pl_ef.bin'.encode('hex')+'00000000')
    '''
    cmd_sec.append(t0 + 0.0)
    cmd.append('pobc_on')
    par.append('')
    cmd_sec.append(t0 + 90.0)
    cmd.append('enable_saving_kpack')
    par.append('')
    cmd_sec.append(t0 + 92.0)
    cmd.append('enable_saving_telemetry')
    par.append('')
    cmd_sec.append(t0 + 94.0)
    cmd.append('enable_saving_status')
    par.append('')
    '''
    cmd_sec.append(t0 + 1.0)
    cmd.append('enable_saving_telemetry')
    par.append('')

    cmd_sec.append(t0 + 2.0)
    cmd.append('set_saving_period_10s')
    par.append('')

    cmd_sec.append(t0 + 3.0)
    cmd.append('start_saving_telemetry_emmc')
    par.append('')
    
    
    cmd_sec.append(t0)
    cmd.append('load_bin_file')
    par.append('706c5f72666c617368332e62696e0000')
    cmd_sec.append(t0 + 2500.0)
    cmd.append('clear_flash')
    par.append('')
    
    cmd_sec.append(t0)
    cmd.append('enable_saving_kpack')
    par.append('')

    cmd_sec.append(t0 + 1.0)
    cmd.append('enable_saving_telemetry')
    par.append('')

    cmd_sec.append(t0 + 2.0)
    cmd.append('set_saving_period_10s')
    par.append('')

    cmd_sec.append(t0 + 3.0)
    cmd.append('start_saving_telemetry_emmc')
    par.append('')
    '''
    # status if it is sun tracking now
    sun_tracking_now = True
    n_cmd = len(att_time_index)
    for i in range(n_cmd):
        j = att_time_index[i]
        q = att_qparms[i]

        t_cmd = t[j]
        if np.size(q) == 4:
            # print("%s, %0.8f, %0.8f, %0.8f, % 0.8f" % (orb_time_bj[j].iso, q[0], q[1], q[2], q[3]))
            # q_dec_str = str(" %0.6f %0.6f %0.6f" % (q[0], q[1], q[2]))
            q_hex_str = struct.pack('>f', q[0]).encode('hex') + \
                        struct.pack('>f', q[1]).encode('hex') + \
                        struct.pack('>f', q[2]).encode('hex')

            if sun_tracking_now:
                if j < test_t:
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new - 4.0)
                    cmd.append('upload_quaternion')
                    par.append(q_hex_str)
                    '''
                    # do twice
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner - 2.0)
                    cmd.append('upload_quaternion')
                    par.append(q_hex_str)
                    '''
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new - 1.0)
                    cmd.append('set_inertial_pointing_mode')
                    par.append('')
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new)
                    cmd.append('start_inertial_pointing')
                    par.append('')
                else:
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new - 4.0)
                    cmd.append('upload_quaternion')
                    par.append(q_hex_str)
                    '''
                    # do twice
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new - 2.0)
                    cmd.append('upload_quaternion')
                    par.append(q_hex_str)
                    '''
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new - 1.0)
                    cmd.append('set_inertial_pointing_mode')
                    par.append('')
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new)
                    cmd.append('start_inertial_pointing')
                    par.append('')
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new + 10.)
                    cmd.append('gyro_lowpassfilter_on')
                    par.append('')
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new + 1690.)
                    cmd.append('temp1')
                    par.append('')
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new + 1695.)
                    cmd.append('temp2')
                    par.append('')
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new + 1700.)
                    cmd.append('enable_pi_control')
                    par.append('')
                    cmd_sec.append(t[j] + sec_delay_from_sun_to_iner_new + 1800.)
                    cmd.append('disable_mwheel_unload')
                    par.append('')

                sun_tracking_now = False
            else:
                cmd_sec.append(t[j])
                cmd.append('upload_quaternion')
                par.append(q_hex_str)
                '''
                # do twice
                cmd_sec.append(t[j] + 2.0)
                cmd.append('upload_quaternion')
                par.append(q_hex_str)
                '''
        else:
            # print("%s, Sun tracking" % (orb_time_bj[j].iso))
            '''
            ##### for pl power keeps on test #### 
            if i == 0:# the first Sun tracking
                cmd.append('wheel_sun_tracking')
                par.append('')
                cmd_sec.append(t0 + 5.0)
            elif att_time_index[i-1] < 3600:
                cmd.append('wheel_sun_tracking')
                par.append('')
                cmd_sec.append(t[j])
            elif att_time_index[i-1] < test_t:
                cmd.append('magnetic_sun_tracking')
                par.append('')
                cmd_sec.append(t[j])
            ########################################
            '''
            if i == 0:# the first Sun tracking
                '''
                cmd.append('start_sun_tracking_mode')
                par.append('')
                cmd_sec.append(t0 + 1005.0)
                '''
                pass
            elif att_time_index[i-1] < test_t:
                cmd.append('start_sun_tracking_mode')
                par.append('')
                cmd_sec.append(t[j])
            else:
                cmd_sec.append(t[j])
                cmd.append('disable_pi_control')
                par.append('')
                cmd_sec.append(t[j] + 2.0)
                cmd.append('enable_mwheel_unload')
                par.append('')
                cmd_sec.append(t[j] + 5.0)
                cmd.append('temp3')
                par.append('')
                cmd_sec.append(t[j] + 8.0)
                cmd.append('temp4')
                par.append('')
                cmd_sec.append(t[j] + 10.0)
                cmd.append('wheel_sun_tracking')
                par.append('')
                
            sun_tracking_now = True
            '''
            if (i != 0 and i!=n_cmd-1 and i%2 == 0): #!!!need to be modified
                cmd_sec.append(t[j] + 2.0)
                cmd.append('reboot_pobc')
                par.append('')
                #print '!!!'
            '''
            # end of the day
            if i == n_cmd - 1:
                ''' failed upgrade in 2019 Dec
                cmd_sec.append(t.max() - 2200.0)
                cmd.append('reboot_pobc')
                par.append('')
                cmd_sec.append(t.max() - 2004.0)
                cmd.append('disable_auto_tg_read')
                par.append('')
                cmd_sec.append(t.max() - 2003.0)
                cmd.append('stop_saving_telemetry_emmc')
                par.append('')
                cmd_sec.append(t.max() - 2002.0)
                cmd.append('disable_saving_kpack')
                par.append('')
                cmd_sec.append(t.max() - 2001.0)
                cmd.append('disable_saving_telemetry')
                par.append('')
                cmd_sec.append(t.max() - 2000.0)
                cmd.append('load_bin_file')
                par.append('706c5f72666c617368332e62696e0000')
                #cmd_sec.append(t.max() - 3503.0)
                #cmd.append('load_bin_file')
                #par.append('706c5f72666c617368342e62696e0000')
                
                cmd_sec.append(t.max() - 1003.0)
                cmd.append('clear_flash')
                par.append('')
                
                cmd_sec.append(t[j] + 2.0)
                cmd.append('reboot_pobc')
                par.append('')
                
                cmd_sec.append(t.max() - 2506.0)
                cmd.append('disable_auto_tg_read')
                par.append('')
                
                cmd_sec.append(t.max() - 3508.0)
                cmd.append('stop_saving_telemetry_emmc')
                par.append('')
                cmd_sec.append(t.max() - 3507.0)
                cmd.append('disable_saving_kpack')
                par.append('')
                cmd_sec.append(t.max() - 3506.0)
                cmd.append('disable_saving_telemetry')
                par.append('')
                
                cmd_sec.append(t.max() - 2502.0)
                cmd.append('adjust_reading_speed_1')
                par.append('')
                cmd_sec.append(t.max() - 2501.0)
                cmd.append('adjust_reading_speed_2')
                par.append('')
                
                cmd_sec.append(t.max() - 3503.0)
                cmd.append('load_bin_file')
                par.append('ty_pl_rf032.bin'.encode('hex')+'00')
                
                cmd_sec.append(t.max() - 3503.0)
                cmd.append('load_bin_file')
                par.append('706c5f72666c617368342e62696e0000')
                #cmd_sec.append(t.max() - 3503.0)
                #cmd.append('load_bin_file')
                #par.append('706c5f72666c617368342e62696e0000')
                
                cmd_sec.append(t.max() - 3503.0)
                cmd.append('load_bin_file')
                par.append('ty_pl_rf3264.bin'.encode('hex'))
                
                '''
                cmd_sec.append(t.max() - 9.0)
                cmd.append('load_bin_file_0s')
                par.append(str("tg_TXDataOff.bin").encode('hex'))
                
                if det_on != 0:
                    cmd_sec.append(t.min() + 7*3600. + 600)
                    cmd.append('magnetic_sun_tracking')
                    par.append('')
                
                cmd_sec.append(t.max() - 5.0)
                cmd.append('disable_saving_status')
                par.append('')
                cmd_sec.append(t.max() - 3.0)
                cmd.append('disable_saving_telemetry')
                par.append('')
                
                cmd_sec.append(t.max() - 1.0)
                cmd.append('disable_saving_kpack')
                par.append('')
                
                
                
                

    cmd_time_bj = Time(cmd_sec, format='unix', scale='utc')
    # for i in range(len(cmd)):
    #     print cmd_time_bj[i].iso, cmd[i]

    return cmd_time_bj, cmd, par, Time(t0, format='unix', scale='utc')

def generate_det_cmd(orb_time_bj, i_target_on, i_target_off, bkg_hv_on, bkg_hv_off):
    cmd = []
    par = []
    temp_time_index = []
    t = orb_time_bj.unix
    #int_sec = t[i_target_off] - t[i_target_on]
    #int_min = np.floor(int_sec / 60.0)
    n = len(i_target_on) + len(bkg_hv_on)
    ttemp_on = i_target_on[:]
    ttemp_on.extend(bkg_hv_on)
    ttemp_on.sort()
    ttemp_off = i_target_off[:]
    ttemp_off.extend(bkg_hv_off)
    ttemp_off.sort()
    #cmd_time_bj = orb_time_bj[i_target_on]
    '''
    cmd_time_bj_on = orb_time_bj[i_target_on]
    cmd_time_bj_off = orb_time_bj[i_target_off]
    for i in range(n):
        cmd.append('load_bin_file_0s')
        
        if int_min[i]%2 == 0:
            int_min[i] = int_min[i]-1
        
        bin_str_name = str("tg_PowerOff.bin" % int_min[i])
        bin_hex_name = bin_str_name.encode('hex')+'0000'
        par.append(bin_hex_name)
    
    ##### for pl power keeps on test #### 
    for i in range(6):
        cmd.append('load_bin_file')
        bin_str_name = str("pl_3kV_%02d.bin" % int_min[i])
        bin_hex_name = bin_str_name.encode('hex')+'000000'
        par.append(bin_hex_name)
        
    for i in range(6,n):
        cmd.append('load_bin_file')
        bin_str_name = str("pl_3kV_%02dmin.bin" % int_min[i])
        bin_hex_name = bin_str_name.encode('hex')
        par.append(bin_hex_name)
    '''
    '''
    # manual
    temp_time_index.append(i_target_on[0])
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_PowerOn.bin")
    bin_hex_name = bin_str_name.encode('hex')+'0000'
    par.append(bin_hex_name)
    temp_time_index.append(i_target_off[0]-1)
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_PowerOff.bin")
    bin_hex_name = bin_str_name.encode('hex')+'00'
    par.append(bin_hex_name)
    
    temp_time_index.append(i_target_on[1])
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_PowerOn.bin")
    bin_hex_name = bin_str_name.encode('hex')+'0000'
    par.append(bin_hex_name)
    temp_time_index.append(i_target_off[1]-1)
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_BiasOff.bin")
    bin_hex_name = bin_str_name.encode('hex')+'0000'
    par.append(bin_hex_name)
    
    temp_time_index.append(i_target_on[2])
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_PowerOn.bin")
    bin_hex_name = bin_str_name.encode('hex')+'0000'
    par.append(bin_hex_name)
    temp_time_index.append(i_target_off[2]-1)
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_PowerOff.bin")
    bin_hex_name = bin_str_name.encode('hex')+'00'
    par.append(bin_hex_name)
    
    temp_time_index.append(i_target_on[3])
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_PowerOn.bin")
    bin_hex_name = bin_str_name.encode('hex')+'0000'
    par.append(bin_hex_name)
    temp_time_index.append(i_target_off[3]-1)
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_BiasOff.bin")
    bin_hex_name = bin_str_name.encode('hex')+'0000'
    par.append(bin_hex_name)
    
    # manual 
    temp_time_index.append(bkg_hv_on[0])
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_IVScan.bin")
    bin_hex_name = bin_str_name.encode('hex')+'000000'
    par.append(bin_hex_name)
    
    temp_time_index.append(bkg_hv_on[1])
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_BiasOn.bin")
    bin_hex_name = bin_str_name.encode('hex')+'000000'
    par.append(bin_hex_name)
    temp_time_index.append(bkg_hv_off[1]-1)
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_PowerOff.bin")
    bin_hex_name = bin_str_name.encode('hex')+'00'
    par.append(bin_hex_name)
    
    temp_time_index.append(bkg_hv_on[2])
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_CI_Obs05m.bin")
    bin_hex_name = bin_str_name.encode('hex')
    par.append(bin_hex_name)
    
    temp_time_index.append(bkg_hv_on[3])
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_BiasOn.bin")
    bin_hex_name = bin_str_name.encode('hex')+'000000'
    par.append(bin_hex_name)
    temp_time_index.append(bkg_hv_off[3]-1)
    cmd.append('load_bin_file_0s')
    bin_str_name = str("tg_PowerOff.bin")
    bin_hex_name = bin_str_name.encode('hex')+'00'
    par.append(bin_hex_name)
     
    for i in range(len(i_target_on)):
        temp_time_index.append(i_target_on[i])
        cmd.append('load_bin_file_0s')
        bin_str_name = str("tg_PowerOn.bin")
        bin_hex_name = bin_str_name.encode('hex')+'0000'
        par.append(bin_hex_name)
        temp_time_index.append(i_target_off[i]-1)
        cmd.append('load_bin_file_0s')
        bin_str_name = str("tg_PowerOff.bin")
        bin_hex_name = bin_str_name.encode('hex')+'00'
        par.append(bin_hex_name)
       
    for i in range(len(bkg_hv_on)):
        temp_time_index.append(bkg_hv_on[i])
        cmd.append('load_bin_file_0s')
        bin_str_name = str("tg_PowerOn.bin")
        bin_hex_name = bin_str_name.encode('hex')+'0000'
        par.append(bin_hex_name)
        temp_time_index.append(bkg_hv_off[i]-1)
        cmd.append('load_bin_file_0s')
        bin_str_name = str("tg_PowerOff.bin")
        bin_hex_name = bin_str_name.encode('hex')+'00'
        par.append(bin_hex_name)
    '''
    for i in range(n):
        #print i
        
        if i>0:
            temp_time_index.append(ttemp_on[i]-6)
            cmd.append('load_bin_file_0s')
            bin_str_name = str("tg_TXDataOff.bin")
            bin_hex_name = bin_str_name.encode('hex')
            par.append(bin_hex_name)
            #print (ttemp_on[i])
        
        if i>0:
            temp_time_index.append(ttemp_on[i])
            cmd.append('load_bin_file_0s')
            bin_str_name = str("tg_PowerOn.bin")
            bin_hex_name = bin_str_name.encode('hex')+'0000'
            par.append(bin_hex_name)
        else:
            temp_time_index.append(ttemp_on[i]+30)
            cmd.append('load_bin_file_0s')
            bin_str_name = str("tg_PowerOnM.bin")
            bin_hex_name = bin_str_name.encode('hex')+'00'
            par.append(bin_hex_name)
            
        temp_time_index.append(ttemp_off[i]-1)
        cmd.append('load_bin_file_0s')
        bin_str_name = str("tg_TXDataOn.bin")
        bin_hex_name = bin_str_name.encode('hex')+'00'
        par.append(bin_hex_name)
    
    cmd_time_bj = orb_time_bj[temp_time_index]
    
    return cmd_time_bj, cmd, par

# return the cmd code given the string
def get_cmd_code(str_cmd):
    cmd_list = {
        'magnetic_sun_tracking':        'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 34 00 14 00 00 00 00 00 08 00 00 00 00 34 08 aa aa',
        #'wheel_sun_tracking':           'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 34 00 05 00 00 00 00 00 08 00 00 00 00 34 02 aa aa',
        'enable_saving_kpack':          'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 02 00 14 00 00 00 00 00 0e 00 00 00 00 03 02 00 0a ff ff ff ff ff ff',
        'disable_saving_kpack':         'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 02 00 14 00 00 00 00 00 0e 00 00 00 00 04 02 00 00 00 00 ff ff ff ff',
        'pobc_on':                      'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 00 00 14 00 00 00 00 00 07 00 00 00 00 03 00 aa',
        'enable_saving_telemetry':      'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 02 00 14 00 00 00 00 00 0e 00 00 00 00 03 34 00 01 ff ff ff ff ff ff',
        'disable_saving_telemetry':     'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 02 00 14 00 00 00 00 00 0e 00 00 00 00 04 34 00 00 00 00 ff ff ff ff',
        'enable_saving_status':         'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 02 00 14 00 00 00 00 00 0e 00 00 00 00 03 00 00 01 ff ff ff ff ff ff',
        'disable_saving_status':        'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 02 00 14 00 00 00 00 00 0e 00 00 00 00 04 00 00 00 00 00 ff ff ff ff',
        #'start_saving_telemetry_emmc':  'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 11 03 00 05 00 00 00 00 00 0c 00 00 00 00 05 49 01 55 55 55 55 55',
        #'stop_saving_telemetry_emmc':   'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 11 03 00 05 00 00 00 00 00 0c 00 00 00 00 05 49 00 55 55 55 55 55',
        #'set_saving_period_10s':        'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 11 03 00 05 00 00 00 00 00 0c 00 00 00 00 05 45 00 00 00 0a 55 55',
        'upload_quaternion':            'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 37 00 14 00 00 00 00 00 11 00 00 00 00 08',
        'set_inertial_pointing_mode':   'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 33 00 14 00 00 00 00 00 08 00 00 00 00 33 29 00 aa',
        'start_inertial_pointing':      'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 34 00 14 00 00 00 00 00 08 00 00 00 00 34 07 aa aa',
        'load_bin_file_0s':             'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 00 00 14 00 02 00 00 00 15 00 00 00 00 04',
        'start_sun_tracking_mode':      'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 34 00 14 00 00 00 00 00 08 00 00 00 00 34 04 aa aa'
        #'target_tracking':              'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 0b 00 05 00 00 00 00 00 04 00 00 00 00',
        #'reboot_pobc':                  'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 c2 00 05 00 00 00 00 00 14 00 00 00 00 70 72 65 73 74 2e 62 69 6e 00 00 00 00 00 00 00',
        #'clear_flash':                  'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 01 c2 00 05 00 00 00 00 00 14 00 00 00 00 74 79 5f 70 6c 5f 65 66 2e 62 69 6e 00 00 00 00',
        #'gyro_lowpassfilter_on':        'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 33 00 05 00 00 00 00 00 08 00 00 00 00 33 18 01 aa',
        #'gyro_lowpassfilter_off':       'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 33 00 05 00 00 00 00 00 08 00 00 00 00 33 18 00 aa',
        #'enable_pi_control':            'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 34 00 05 00 00 00 00 00 08 00 00 00 00 34 1e 01 aa',
        #'disable_pi_control':           'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 34 00 05 00 00 00 00 00 08 00 00 00 00 34 1e 00 aa',
        #'enable_mwheel_unload':         'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 34 00 05 00 00 00 00 00 08 00 00 00 00 34 20 01 aa',
        #'disable_mwheel_unload':        'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 34 00 05 00 00 00 00 00 08 00 00 00 00 34 20 00 aa',
        #'temp1':                        'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 36 00 05 00 00 00 00 00 08 00 00 00 00 36 01 25 aa',
        #'temp2':                        'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 36 00 05 00 00 00 00 00 08 00 00 00 00 36 02 1b aa',
        #'temp3':                        'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 36 00 05 00 00 00 00 00 08 00 00 00 00 36 01 00 aa',
        #'temp4':                        'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 7e 36 00 05 00 00 00 00 00 08 00 00 00 00 36 02 00 aa',
        #'disable_auto_tg_read':         'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 11 03 00 05 00 00 00 00 00 0c 00 00 00 00 05 06 00 00 00 00 00 00',
        #'adjust_reading_speed_1':       'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 11 03 00 05 00 00 00 00 00 0c 00 00 00 00 05 54 01 00 00 00 00 00',
        #'adjust_reading_speed_2':       'eb 90 01 00 01 01 00 00 00 00 00 00 01 00 11 03 00 05 00 00 00 00 00 0c 00 00 00 00 05 52 01 00 00 00 00 00'
    }

    if str_cmd in cmd_list.keys():
        txt_cmd = cmd_list[str_cmd]
        '''
        if str_cmd.startswith('load_bin_file'):
            txt_cmd += '00000000'
        '''
    else:
        txt_cmd = 'unknown_cmd'

    return txt_cmd.replace(' ', '')

# given hex string and calculate 4 byte CRC
def calc_crc(hex_str):
    crc_tab = np.array([
        0x00000000, 0xF26B8303, 0xE13B70F7, 0x1350F3F4, 0xC79A971F, 0x35F1141C, 0x26A1E7E8, 0xD4CA64EB,
        0x8AD958CF, 0x78B2DBCC, 0x6BE22838, 0x9989AB3B, 0x4D43CFD0, 0xBF284CD3, 0xAC78BF27, 0x5E133C24,
        0x105EC76F, 0xE235446C, 0xF165B798, 0x030E349B, 0xD7C45070, 0x25AFD373, 0x36FF2087, 0xC494A384,
        0x9A879FA0, 0x68EC1CA3, 0x7BBCEF57, 0x89D76C54, 0x5D1D08BF, 0xAF768BBC, 0xBC267848, 0x4E4DFB4B,
        0x20BD8EDE, 0xD2D60DDD, 0xC186FE29, 0x33ED7D2A, 0xE72719C1, 0x154C9AC2, 0x061C6936, 0xF477EA35,
        0xAA64D611, 0x580F5512, 0x4B5FA6E6, 0xB93425E5, 0x6DFE410E, 0x9F95C20D, 0x8CC531F9, 0x7EAEB2FA,
        0x30E349B1, 0xC288CAB2, 0xD1D83946, 0x23B3BA45, 0xF779DEAE, 0x05125DAD, 0x1642AE59, 0xE4292D5A,
        0xBA3A117E, 0x4851927D, 0x5B016189, 0xA96AE28A, 0x7DA08661, 0x8FCB0562, 0x9C9BF696, 0x6EF07595,
        0x417B1DBC, 0xB3109EBF, 0xA0406D4B, 0x522BEE48, 0x86E18AA3, 0x748A09A0, 0x67DAFA54, 0x95B17957,
        0xCBA24573, 0x39C9C670, 0x2A993584, 0xD8F2B687, 0x0C38D26C, 0xFE53516F, 0xED03A29B, 0x1F682198,
        0x5125DAD3, 0xA34E59D0, 0xB01EAA24, 0x42752927, 0x96BF4DCC, 0x64D4CECF, 0x77843D3B, 0x85EFBE38,
        0xDBFC821C, 0x2997011F, 0x3AC7F2EB, 0xC8AC71E8, 0x1C661503, 0xEE0D9600, 0xFD5D65F4, 0x0F36E6F7,
        0x61C69362, 0x93AD1061, 0x80FDE395, 0x72966096, 0xA65C047D, 0x5437877E, 0x4767748A, 0xB50CF789,
        0xEB1FCBAD, 0x197448AE, 0x0A24BB5A, 0xF84F3859, 0x2C855CB2, 0xDEEEDFB1, 0xCDBE2C45, 0x3FD5AF46,
        0x7198540D, 0x83F3D70E, 0x90A324FA, 0x62C8A7F9, 0xB602C312, 0x44694011, 0x5739B3E5, 0xA55230E6,
        0xFB410CC2, 0x092A8FC1, 0x1A7A7C35, 0xE811FF36, 0x3CDB9BDD, 0xCEB018DE, 0xDDE0EB2A, 0x2F8B6829,
        0x82F63B78, 0x709DB87B, 0x63CD4B8F, 0x91A6C88C, 0x456CAC67, 0xB7072F64, 0xA457DC90, 0x563C5F93,
        0x082F63B7, 0xFA44E0B4, 0xE9141340, 0x1B7F9043, 0xCFB5F4A8, 0x3DDE77AB, 0x2E8E845F, 0xDCE5075C,
        0x92A8FC17, 0x60C37F14, 0x73938CE0, 0x81F80FE3, 0x55326B08, 0xA759E80B, 0xB4091BFF, 0x466298FC,
        0x1871A4D8, 0xEA1A27DB, 0xF94AD42F, 0x0B21572C, 0xDFEB33C7, 0x2D80B0C4, 0x3ED04330, 0xCCBBC033,
        0xA24BB5A6, 0x502036A5, 0x4370C551, 0xB11B4652, 0x65D122B9, 0x97BAA1BA, 0x84EA524E, 0x7681D14D,
        0x2892ED69, 0xDAF96E6A, 0xC9A99D9E, 0x3BC21E9D, 0xEF087A76, 0x1D63F975, 0x0E330A81, 0xFC588982,
        0xB21572C9, 0x407EF1CA, 0x532E023E, 0xA145813D, 0x758FE5D6, 0x87E466D5, 0x94B49521, 0x66DF1622,
        0x38CC2A06, 0xCAA7A905, 0xD9F75AF1, 0x2B9CD9F2, 0xFF56BD19, 0x0D3D3E1A, 0x1E6DCDEE, 0xEC064EED,
        0xC38D26C4, 0x31E6A5C7, 0x22B65633, 0xD0DDD530, 0x0417B1DB, 0xF67C32D8, 0xE52CC12C, 0x1747422F,
        0x49547E0B, 0xBB3FFD08, 0xA86F0EFC, 0x5A048DFF, 0x8ECEE914, 0x7CA56A17, 0x6FF599E3, 0x9D9E1AE0,
        0xD3D3E1AB, 0x21B862A8, 0x32E8915C, 0xC083125F, 0x144976B4, 0xE622F5B7, 0xF5720643, 0x07198540,
        0x590AB964, 0xAB613A67, 0xB831C993, 0x4A5A4A90, 0x9E902E7B, 0x6CFBAD78, 0x7FAB5E8C, 0x8DC0DD8F,
        0xE330A81A, 0x115B2B19, 0x020BD8ED, 0xF0605BEE, 0x24AA3F05, 0xD6C1BC06, 0xC5914FF2, 0x37FACCF1,
        0x69E9F0D5, 0x9B8273D6, 0x88D28022, 0x7AB90321, 0xAE7367CA, 0x5C18E4C9, 0x4F48173D, 0xBD23943E,
        0xF36E6F75, 0x0105EC76, 0x12551F82, 0xE03E9C81, 0x34F4F86A, 0xC69F7B69, 0xD5CF889D, 0x27A40B9E,
        0x79B737BA, 0x8BDCB4B9, 0x988C474D, 0x6AE7C44E, 0xBE2DA0A5, 0x4C4623A6, 0x5F16D052, 0xAD7D5351
    ])

    crc = 0xFFFFFFFF

    n = len(hex_str)
    if n == 0:
        return crc

    for i in range(0, n, 2):
        byte = hex_str[i:i+2]
        crc = crc_tab[(crc ^ int('0x' + byte, 0)) & 0xFFL] ^ (crc >> 8)

    crc_hex = str("%08x" % (crc ^ 0xFFFFFFFF))
    return crc_hex

def combine_and_generate_txt_cmd(att_cmd_time_bj, att_cmd, att_par, det_cmd_time_bj, det_cmd, det_par, t0_bj):
    # combine the two cmd list
    all_unix_bj = np.concatenate([att_cmd_time_bj.unix, det_cmd_time_bj.unix])
    all_cmd = np.array(att_cmd + det_cmd)
    all_par = np.array(att_par + det_par)
    # sort by time
    q = np.argsort(all_unix_bj)
    unix_bj = all_unix_bj[q]
    str_cmd = all_cmd[q]
    hex_par = all_par[q]

    # convert from string command to hex-txt command
    hex_cmd = []
    for i in range(len(str_cmd)):
        ss = get_cmd_code(str_cmd[i]) + hex_par[i]
        ss += calc_crc(ss)
        hex_cmd.append(ss)
    
    # generate the time for the txt command file, relative time within 120s, otherwise absolute time
    sec_elapse = unix_bj - t0_bj.unix # seconds since T0
    #q = sec_elapse <= 120.0
    #sec_within_120 = np.diff(np.insert(sec_elapse[q], 0, 0.))
    #sec_after_120 = sec_elapse[~q]
    # convert to s
    s_all = np.round(sec_elapse)
    #print s_all
    # convert to s
    txt_s = [str("%08x" % x) for x in s_all]

    # write the txt file into disk
    txt_filename = 'tg_' + (t0_bj.isot[0:-4]).replace('-', '').replace(':', 'h', 1).replace(':', 'm', 1) + 's.txt'
    ftxt = open(txt_filename, 'w')

    # write the bin file
    bin_filename = txt_filename.replace('.txt', '.bin')
    fbin = open(bin_filename, 'wb')
    # version control
    fbin.write('UF20')

    for i in range(len(str_cmd)):
        strpar = ''
        if str_cmd[i].startswith('load_bin_file'):
            strpar = ' ' + hex_par[i].decode('hex')
        if str_cmd[i].startswith('upload_quaternion'):
            q0 = struct.unpack('>f', hex_par[i][0:8].decode('hex'))[0]
            q1 = struct.unpack('>f', hex_par[i][8:16].decode('hex'))[0]
            q2 = struct.unpack('>f', hex_par[i][16:24].decode('hex'))[0]
            strpar = str(" %0.6f %0.6f %0.6f" % (q0, q1, q2))
        # comment line
        ftxt.write("#%02d, %s, %s\n" % (i+1, Time(unix_bj[i], format='unix', scale='utc').iso[0:-4], str_cmd[i] + strpar))
        # hex txt cmd line
        cmd_line = txt_s[i] + hex_cmd[i]
        #print cmd_line
        ftxt.write("%s\n" % insert_2blanck(cmd_line))
        # print cmd_line
        fbin.write(cmd_line.decode('hex'))

    # close the files
    ftxt.close()
    fbin.close()
'''
def from_sun_pos_get_grid_pos(time):
    sun_ra = get_sun(time).ra.rad
    sun_dec = get_sun(time).dec.rad
    sun_pos_xyz = np.array([pl.radec_to_xyz(sun_ra, sun_dec)])
    sun_in_grid_ra = 4.5
    sun_in_grid_dec = 0.5
    sun_pos_in_grid_xyz = np.array([pl.radec_to_xyz(sun_in_grid_ra, sun_in_grid_dec)])
    rot_m = np.dot(sun_pos_in_grid_xyz.T, sun_pos_xyz)
    y = np.array([[0, 1, 0]])
    st = np.array([[0, 0, 1]])
    y_t = np.matmul(y, rot_m)
    st_t = np.matmul(st, rot_m)
    y_ra, y_dec = pl.xyz_to_radec(y_t[0][0], y_t[0][1], y_t[0][2])
'''   



# insert a space between every two characters
def insert_2blanck(sin):
    pattern = re.compile('.{2}')
    sout = ' '.join(pattern.findall(sin))
    return sout
########################################################################
########################################################################
########################################################################

TARGET_SUN_AVOID_ANGLE = np.deg2rad(20.0)
MIN_HV_ON_TIME = 20.0 * 60.0 # in seconds

args = sys.argv

if len(args) > 2:
    target_name = args[1]
    orbit_file = args[2] # file for the orbit lon, lat, height
else:
    print("Usage:")
    print("   obs_schedule target /path/orb.txt")
    print("   target = Crab/Scox1")
    sys.exit()

if 'doplot' in args:
    doplot = True
else:
    doplot = False

if 'reflash' in args:
    reflash = True
else:
    reflash = False

if 'bkg' in args:
    bkg = True
else:
    bkg = True

if 'grid' in args:
    grid = True
else:
    grid = False
    
if 'det_on' in args:
    det_on = 40
else:
    det_on = 0

if target_name.lower() == 'crab':
    # target coordinates
    target_ra = np.deg2rad(083.63308)
    target_dec = np.deg2rad(22.01450)
elif target_name.lower() == 'scox1':
    target_ra = np.deg2rad(244.9794536441080)
    target_dec = np.deg2rad(-15.6402833113192)
elif target_name.lower() == 'cygx1':
    target_ra = np.deg2rad(299.590)
    target_dec = np.deg2rad(35.202)
elif target_name.lower() == 'gx5-1':
    target_ra = np.deg2rad(270.284)
    target_dec = np.deg2rad(-25.079)
elif target_name.lower() == 'velax1':
    target_ra = np.deg2rad(135.529)
    target_dec = np.deg2rad(-40.555)
elif target_name.lower() == 'cenx3':
    target_ra = np.deg2rad(170.315)
    target_dec = np.deg2rad(-60.623)
elif target_name.lower() == 'a0535+262':
    target_ra = np.deg2rad(84.727)
    target_dec = np.deg2rad(26.316)
else:
    print("Unknown target: %s" % target_name)
    sys.exit()

########################################################################
saa_coord_file = '../../saa/coords.txt'
saa_flux_file = '../../saa/AE8_MIN_0.1MeV.txt'

# 'Altitude','km', 1,'Altitude'
# 'Latitude','deg', 1,'Latitude'
# 'Longitude','deg', 1,'Longitude'
# 'LT','hr', 1,'Local Time'
# 'UT','hr', 1,'Universal Time'
coord = np.loadtxt(saa_coord_file, comments="'", skiprows=26, delimiter=',')
map_lat = np.deg2rad(coord[:, 1])
map_lon = np.deg2rad(coord[:, 2])

# 'B','Gauss', 1,'!8B!3'
# 'L','!8R!3!dE!n', 1,'!8L!3'
# 'Flux','cm!u-2!n s!u-1!n', 1,'AE-8 MIN Flux >   0.10 MeV'
flux = np.loadtxt(saa_flux_file, comments="'", skiprows=30, delimiter=',')[:, 2]
flux[flux <= 0] = 1e-100

n_lon = 121
n_lat = 90
########################################################################
orb_time_bj, orb_lat_deg, orb_lon_deg, orb_alt, orb_ra_deg, orb_dec_deg, = pl.read_STK_orbit_file(orbit_file)
# convert from degree to radian
orb_lat = np.deg2rad(orb_lat_deg)
orb_lon = np.deg2rad(orb_lon_deg)
orb_ra = np.deg2rad(orb_ra_deg)
orb_dec = np.deg2rad(orb_dec_deg)
# convert from BJ time to UTC
orb_time = Time(orb_time_bj.unix - 8. * 3600., format='unix', scale='utc')
orb_mjd = orb_time.mjd
n = len(orb_time)
########################################################################

print("Calculating Sun's position. This may take a while ... "),
sun_pos = get_sun(orb_time)
sun_ra = sun_pos.ra.radian
sun_dec = sun_pos.dec.radian
c_sun = SkyCoord(ra=sun_ra, dec=sun_dec, unit='radian', frame='fk5')
print("done")

earth_ra = orb_ra + np.pi
earth_dec = -orb_dec
sun_angle = pl.angular_distance(sun_ra, sun_dec, target_ra, target_dec)
earth_angle = pl.angular_distance(earth_ra, earth_dec, target_ra, target_dec)

R_earth = 6400.0 # Earth's radius in km
h_atm = 100.0 # Earth's atmosphere thickness in km
h_sat = (orb_alt + R_earth) # satellite height in km
earth_occultation_angle = np.arcsin(1)#np.arcsin((R_earth + h_atm) / h_sat)

################################################################################################
# get the flux along the trajectory
points = np.zeros([n_lat * n_lon, 2])
points[:, 0] = map_lon
points[:, 1] = map_lat
orb_log_flux = interpolate.griddata(points, np.log10(flux), (orb_lon, orb_lat), method='linear')

################################################################################################
# find time intervals
good_hv = orb_log_flux < 1.0
good_visibility = earth_angle > earth_occultation_angle
good_sun_angle = sun_angle > TARGET_SUN_AVOID_ANGLE

tt = orb_time.unix # in units of seconds

# when HV can be ON
i0_hv, i1_hv = pl.find_continous_intervals(good_hv)
t0_hv = tt[i0_hv] # time for HV on
t1_hv = tt[i1_hv] # time for HV off
dt_hv = t1_hv - t0_hv # duration
min_hv_time = MIN_HV_ON_TIME # minimum seconds to power on the detector
q_hv = (dt_hv >= min_hv_time)
# intervals long enough to power on the HV
i_hv_on = np.array(i0_hv)[q_hv]
i_hv_off = np.array(i1_hv)[q_hv]

if len(i_hv_on)>0 and i_hv_on[0] < 50:
    i_hv_on = i_hv_on[1:]
    i_hv_off = i_hv_off[1:]
n_hv = len(i_hv_on)


i_target_on = []
i_target_off = []
for i in range(n_hv):
    # time when HV is ON and OFF
    t0 = tt[i_hv_on[i]]
    t1 = tt[i_hv_off[i]]

    # index for HV on
    q = range(i_hv_on[i], i_hv_off[i] + 1)
    # times when HV is on
    t = tt[q]
    # visibility to the target
    q_visible = (good_visibility & good_sun_angle)[q]
    # find continuous time intervals when the target is visible
    i0_vis, i1_vis = pl.find_continous_intervals(q_visible)
    # find time intervals that are long enough
    t0_vis = t[i0_vis]
    t1_vis = t[i1_vis]
    dt_vis = t1_vis - t0_vis

    # only choose time intervals long enough
    p = dt_vis > MIN_HV_ON_TIME

    for j in range(len(dt_vis)):
        if dt_vis[j] >= MIN_HV_ON_TIME:
            i_target_on.append(i_hv_on[i] + i0_vis[j])
            i_target_off.append(i_hv_on[i] + i1_vis[j])

##
while len(i_target_on)>0 and i_target_on[0] < 20:
    i_target_on = i_target_on[1:]
    i_target_off = i_target_off[1:]
    
while len(i_target_on)>0 and i_target_off[-1] > len(orb_time_bj) - 10:
    i_target_on = i_target_on[:-1]
    i_target_off = i_target_off[:-1]


dt_target = tt[i_target_off] - tt[i_target_on]

dt_target_r = dt_target//60
s = 0
ii = 0
while ii<len(dt_target_r) and s < det_on:   ##### control the time of target observation
    s += (dt_target_r[ii]-7.5)
    ii += 1

#ii = 10

i_target_on = i_target_on[:ii]
i_target_off = i_target_off[:ii]
dt_target = dt_target[:ii]
bkg_hv_on = []
bkg_hv_off = []
if bkg:
    '''
    bkg_hv_on = i_hv_on[i_hv_on > i_target_off[-1]]
    bkg_hv_off = i_hv_off[i_hv_off > i_target_off[-1]]
    '''
    for i in range(len(i_hv_on)):
        ff = True
        for j in range(len(i_target_on)):
            if i_hv_on[i] <= i_target_on[j] and i_hv_off[i] >= i_target_off[j]:
                ff = False
        if ff:
            bkg_hv_on.append(i_hv_on[i])
            bkg_hv_off.append(i_hv_off[i])
    
    if len(bkg_hv_on)==0:
        bkg = False
    else:
        if bkg_hv_off[0] < bkg_hv_on[0]:
            bkg_hv_off = bkg_hv_off[1:]
        while (len(bkg_hv_on)>0 and bkg_hv_off[-1] > len(orb_time_bj) - 1):
            bkg_hv_on = bkg_hv_on[:-1]
            bkg_hv_off = bkg_hv_off[:-1]
            
    if len(bkg_hv_on)==0:
        bkg = False

while len(bkg_hv_on)>0 and bkg_hv_on[0] < 20:
    bkg_hv_on = bkg_hv_on[1:]
    bkg_hv_off = bkg_hv_off[1:]
    
while len(bkg_hv_on)>0 and bkg_hv_off[-1] > len(orb_time_bj) - 10:
    bkg_hv_on = bkg_hv_on[:-1]
    bkg_hv_off = bkg_hv_off[:-1]

dt_bkg = tt[bkg_hv_off] - tt[bkg_hv_on]
n_obs = len(dt_target)
n_bkg = len(dt_bkg)

print("From %s to %s (Beijing time)" % (orb_time_bj[0].iso, orb_time_bj[-1].iso))
print("   Power ON/OFF times: %d" % n_hv)
print("   Total power ON time: %0.2f hours" % (np.sum(dt_hv[q_hv]) / 3600.))
print("   Target: %s" % target_name.upper())
print("   Target observation times: %d" % n_obs)
print("   Total target observation time: %0.2f minutes" % s )
print("   Background observation times: %d" % n_bkg)
print("   Total background observation time: %0.2f hours" % ((np.sum(dt_bkg)-450*n_bkg) / 3600.))


# find star tracker's possible RA/Dec
possible_st_ra, possible_st_dec, possible_st_sun_angle, possible_st_earth_angle = possible_star_tracker_pointings_for_grid_new(target_ra, target_dec, c_sun, earth_ra, earth_dec)
# find the good RA/Dec for the stracker
st_ra, st_dec, st_time_index = select_star_tracker_radec(possible_st_ra, possible_st_dec, possible_st_sun_angle, possible_st_earth_angle, i_target_on, i_target_off)
# calculate the quaternions
qparms = calcualte_qparm(target_ra, target_dec, st_ra, st_dec)

# tracking the sun to charge the battery when no observing
# star to track the sun right after the target off
sun_tracking_time_index = [0] + [j + 1 for j in i_target_off]
if sun_tracking_time_index[-1] == len(orb_time_bj):
    sun_tracking_time_index[-1] -= 1
# combine the inertial pointings and sun-tracking
att_time_index, att_qparms = attitude_assembly(qparms, st_time_index, sun_tracking_time_index)
# get Earth/Sun angle wrt the star tracker
star_tracker_earth_sun_angle(att_qparms, att_time_index, earth_ra, earth_dec, sun_ra, sun_dec)

att_cmd_time_bj, att_cmd, att_par, t0 = generate_att_cmd(orb_time_bj, att_time_index, att_qparms)

det_cmd_time_bj, det_cmd, det_par = generate_det_cmd(orb_time_bj, i_target_on, i_target_off, bkg_hv_on, bkg_hv_off)
'''
temp_time_index = list(i_target_on)

if bkg:
    bkg_cmd_time_bj, bkg_cmd, bkg_par = generate_det_cmd(orb_time_bj, bkg_hv_on, bkg_hv_off)
    temp_time_index = list(i_target_on) #there will be reboot info in it, so it's not a real targer_on time index
    for i in range(n_bkg):
        temp_time_index.append(bkg_hv_on[i])
        det_cmd.append(bkg_cmd[i])
        det_par.append(bkg_par[i])
        
        if i%2==1 or i==len(bkg_hv_on)-1:
            temp_time_index.append(bkg_hv_off[i]+1)
            det_cmd.append('reboot_pobc')
            det_par.append('')
            temp_time_index.append(bkg_hv_off[i]+91)
            det_cmd.append('magnetic_sun_tracking')
            det_par.append('')
'''   
############## data transfer ####################
### change it every time, need to find a way to automatically do it ###
###
'''
data_transfer_time_1 = 19*3600 + 47*60 + 19 - 12*3600 
temp_time_index.append(data_transfer_time_1/10 - 40*6)  
det_cmd.append('wheel_sun_tracking')
det_par.append('')
temp_time_index.append(data_transfer_time_1/10 - 30*6)  
det_cmd.append('reboot_pobc')
det_par.append('')

data_transfer_time_2 = 8*3600 + 26*60 + 39 + 12*3600 #seconds 
temp_time_index.append(data_transfer_time_2/10 - 40*6)  
det_cmd.append('wheel_sun_tracking')
det_par.append('')
temp_time_index.append(data_transfer_time_2/10 - 30*6)  
det_cmd.append('reboot_pobc')
det_par.append('')
'''
#############################
#det_cmd_time_bj = orb_time_bj[temp_time_index]
'''
temp_time_index.append(i_target_off[4] + 1)
det_cmd.append('reboot_pobc')
det_par.append('')
temp_time_index.append(i_target_off[4] + 6)
det_cmd.append('load_bin_file')
det_par.append('ty_pl_rf032.bin'.encode('hex')+'00')
temp_time_index.append(i_target_off[4] + 256)
det_cmd.append('load_bin_file')
det_par.append('ty_pl_ef.bin'.encode('hex')+'00000000')


########## insert grid command ############
if grid:
    grid_times = 10
    grid_start = 720
    for i in range(grid_times):
        det_cmd.append('load_bin_file')
        det_par.append('ty_tg_on.bin'.encode('hex')+'00000000')
        temp_time_index.append(grid_start+i*720)
        det_cmd.append('load_bin_file')
        det_par.append('ty_tg_off.bin'.encode('hex')+'000000')
        temp_time_index.append(grid_start+360+i*720)
        det_cmd.append('reboot_pobc')
        det_par.append('')
        temp_time_index.append(grid_start+361+i*720)
        

###################################  
'''

#det_cmd_time_bj = orb_time_bj[temp_time_index]
combine_and_generate_txt_cmd(att_cmd_time_bj, att_cmd, att_par, det_cmd_time_bj, det_cmd, det_par, t0)

if True:
    fig = plt.figure(figsize=(15, 5))
    ax = fig.add_subplot(111)

    x = (tt - tt[0]) / 60.

    # l1, = ax.plot(x, orb_log_flux + 100., 'lightblue', lw=1)
    # ax.plot(tt, sun_angle)
    l_ea, = ax.plot(x, np.rad2deg(earth_angle), 'gray', lw=1)
    # ax.plot(x, earth_avoid_angle)

    # y0, y1 = ax.get_ylim()
    yr = [0, 200]

    # regions with HV on
    for i in range(n_hv):
        # q = range(i_hv_on[i], i_hv_off[i] + 1)
        # l_hv_on, = ax.plot(x[q], orb_log_flux[q] + 100., 'b', lw=3)
        x0 = x[i_hv_on[i]]
        x1 = x[i_hv_off[i]]
        l_hv, = ax.fill((x0, x1, x1, x0), (yr[0], yr[0], yr[1], yr[1]), 'g', alpha=0.3)

    # get the aspect angle between the Earth/Sun and the star tracker
    st_ea, st_sa = star_tracker_earth_sun_angle(att_qparms, att_time_index, earth_ra, earth_dec, sun_ra, sun_dec)
    for i in range(n_obs):
        q = range(i_target_on[i], i_target_off[i] + 1)
        l_target_on, = ax.plot(x[q], np.rad2deg(earth_angle[q]), 'g', lw=3)
        # plot star tracker earth angle
        l_st_ea, = ax.plot(x[q], np.rad2deg(st_ea[q]), 'b', lw=1)
        l_st_sa, = ax.plot(x[q], np.rad2deg(st_sa[q]), 'r', lw=1)
     
    
    try:
        ax.plot([data_transfer_time_1/60. - 30,data_transfer_time_1/60. - 30],[25,150],'g',lw=1)
    except NameError:
        pass
    try:
        ax.plot([data_transfer_time_2/60. - 30,data_transfer_time_2/60. - 30],[25,150],'g',lw=1)
    except NameError:
        pass
    
    
    if bkg:
        for i in range(n_bkg):
            q = range(bkg_hv_on[i], bkg_hv_off[i] + 1)
            l_bkg_on, = ax.plot(x[q], np.rad2deg(earth_angle[q]), 'c', lw=3)

    if grid:
        for i in range(grid_times):# to be modified
            q = range(grid_start+i*720, grid_start+360+i*720)
            l_grid_on, = ax.plot(x[q], np.ones(len(q))*25, 'r', lw=3)
    '''
    st_ea_grid, st_sa_grid = star_tracker_earth_sun_angle(att_qparms, att_time_index, earth_ra, earth_dec, sun_ra, sun_dec)
    if det_on:
        for i in on_index:
            q = range(i_hv_on[qq][i], i_hv_off[qq][i] + 1)
            l_st_sa_grid, = ax.plot(x[q], np.rad2deg(st_sa_grid[q]), 'r', lw=1)
    '''    
    
    # intervals with sun tracking
    q = (st_sa == 0)
    j0, j1 = pl.find_continous_intervals(q)
    for i in range(len(j0)):
        x0 = x[j0[i]]
        x1 = x[j1[i]]
        l_sf, = ax.fill((x0, x1, x1, x0), (yr[0], yr[0], 90, 90), 'r', alpha=0.1, edgecolor='r', hatch='//')

    #ax.set_xlim([610,660])
    ax.set_ylim(yr)
    ax.set_xlabel('Time (minutes)')
    ax.set_ylabel('Angle (degrees)')
    if bkg and grid:
        ax.legend((l_ea, l_st_ea, l_st_sa, l_hv, l_target_on, l_bkg_on, l_sf, l_grid_on),
              ['Target/Earth angle', 'ST/Earth angle', 'ST/Sun angle', 'HV ON', 'Target ON', 'Background ON', 'Sun tracking', 'GRID on'],
              loc='upper left', ncol=7)
    elif bkg:
        ax.legend((l_ea, l_hv, l_bkg_on, l_sf),
              ['Target/Earth angle', 'HV ON', 'Background ON', 'Sun tracking'],
              loc='upper left', ncol=7)
    elif grid:
        ax.legend((l_ea, l_st_ea, l_st_sa, l_hv, l_target_on, l_sf, l_grid_on),
              ['Target/Earth angle', 'ST/Earth angle', 'ST/Sun angle', 'HV ON', 'Target ON', 'Sun tracking', 'GRID on'],
              loc='upper left', ncol=7)
    else:
        ax.legend((l_ea, l_st_ea, l_st_sa, l_hv, l_target_on, l_sf),
              ['Target/Earth angle', 'ST/Earth angle', 'ST/Sun angle', 'HV ON', 'Target ON', 'Sun tracking'],
              loc='upper left', ncol=7)

    plt.show()
    #plt.scatter(orb_lon[good_hv],orb_lat[good_hv])
    
    plt.figure(figsize=[9,6])
    att_start = i_target_on
    att_stop = i_target_off
    ttt = flux > 1
    plt.scatter(np.rad2deg(map_lon[ttt]),np.rad2deg(map_lat[ttt]),s=1)
    for i in range(n_obs):
        plt.scatter(np.rad2deg(orb_lon[att_start[i]:att_stop[i]]),np.rad2deg(orb_lat[att_start[i]:att_stop[i]]),c='r',s=1)
    
    att_start = bkg_hv_on
    att_stop = bkg_hv_off
    if bkg:
        for i in range(n_bkg):
            plt.scatter(np.rad2deg(orb_lon[att_start[i]:att_stop[i]]),np.rad2deg(orb_lat[att_start[i]:att_stop[i]]),c='m',s=1)
    #ttt1 = 5294
    #ttt2 = 5494
    #plt.scatter(np.rad2deg(orb_lon[ttt1:ttt2]),np.rad2deg(orb_lat[ttt1:ttt2]),c='y',s=1)
    plt.xlabel('longitude')
    plt.ylabel('latitude')
    plt.show()
    

